# ###############
# # DESCRIPTION #
# ###############

# This script creates a framed text from a newline-separated string.
# For more information run it with the -h argument: "create-framed-text.py -h".

# ##################
# # Import Modules #
# ##################

import os
import sys
import getopt
import operator

# #########################################
# # Define Global Constants And Variables #
# #########################################

SCRIPT_NAME =                str(sys.argv[0][sys.argv[0].rfind("\\")+1:]),                         # Name of the script (without path)

MAN =                        [
                             ("NAME",            "This script creates a framed text from a newline-separated string."),
                             ("VER",             "1.0"),
                             ("SYNOPSIS",        "[OPTION]... <input>"),
                             ("DESCRIPTION",     ""),
                             ("AUTHOR",          "Written by Adam Pócs"),
                             ("REPORTING BUGS",  "Report bugs to royaljerry@gmail.com"),
                             ("COPYRIGHT",       "Copyright (C) 2007 Adam Pócs; this is free software.")
                             ]
OPTIONS_SHORT =              [
                             ("h",               "Print this message and exit."),
                             ("f",               "Take input as file."),
                             ("s <number>",      "Horizontal space between text and frame."),
                             ("S <number>",      "Vertical space between text and frame."),
                             ("t",               "Trim leading spaces."),
                             ("T",               "Trim trailing spaces."),
                             ("a <direction>",   "Align text block <direction> (left, right, center)."),
                             ("c <character>",   "Frame character."),
                             ("m <language>",    "Create multiline comment block in <language> (c, python)."),
                             ("v",               "Verbose mode.")
                             ]
OPTIONS_LONG =               [
                             ("regular",         (
                                                 "Trim leading and trailing spaces, align text block left",
                                                 "(same as -tT -a left)."
                                                 )),
                             ("com-c",           (
                                                 "Trim leading and trailing spaces, align text block left",
                                                 "comment out text with c-syntax",
                                                 "(same as -tT -a left -m c)."
                                                 )),
                             ("com-python",      (
                                                 "Trim leading and trailing spaces, align text block left",
                                                 "comment out text with python-syntax",
                                                 "(same as -tT -a left -m python)."
                                                 )),
                             ]

TAB_0 =                      7                                                                     # Indent of the usage sytax descriptions before arguments when requestng help
TAB_1 =                      20                                                                    # Indent of the usage sytax descriptions after arguments when requestng help
INPUT_STRING =               sys.argv[len(sys.argv)-1]                                             # Input string reference
INPUT_STRING_OBJ =           ""                                                                    # Input string reference as object
INPUT_STRING_LINES =         []                                                                    # List of input string lines
OUTPUT_STRING =              ""                                                                    # Output string
OUTPUT_LINES =               []                                                                    # List of output string lines

O_HELP =                     False                                                                 # Print this message and exit
O_FILE =                     False                                                                 # Take input as file
O_SPACE_HOR =                1                                                                     # Horizontal space between text and frame
O_SPACE_VER =                0                                                                     # Vertical space between text and frame
O_TRIM_L =                   False                                                                 # Trim leading spaces
O_TRIM_T =                   False                                                                 # Trim trailing spaces
O_ALIGN =                    "left"                                                                # Align text block <direction> (left, right, center)
O_FRAME =                    "#"                                                                   # Frame character <character>
O_COMMENT =                  False                                                                 # Create multiline comment block in <language> (C, Python)
O_VERBOSE =                  False                                                                 # Verbose mode

# #############
# # Functions #
# #############

def printUsage(sName, man, oShort, oLong, indent0, indent1):
	# Init vars for the text to be printed
	cPrint = ""
	fChar = " "
	# Print sections and their descriptions
	for i in range(len(man)):
		nMan = man[i][0]
		if nMan == "NAME": dMan = sName + ": " + man[i][1]
		elif nMan == "SYNOPSIS": dMan = sName + " " + man[i][1]
		else: dMan = man[i][1]
		if nMan == "DESCRIPTION":
			cPrint += nMan
			# Print short descriptions
			for opt in range(len(oShort)):
				cPrint += "\n\n"
				cLine = str(fChar*TAB_0) + "-" + oShort[opt][0].ljust(indent1, " ") + oShort[opt][1]
				cPrint += cLine
			# Print long descriptions
			for opt in range(len(oLong)):
				cPrint += "\n\n"
				cLine = ""
				for desc in range(len(oLong[opt][1])):
					if desc == 0: cLine = str(fChar*TAB_0) + "--" + oLong[opt][0].ljust(indent1-1, " ") + oLong[opt][1][desc] + "\n"
					elif desc < len(oLong[opt][1])-1: cLine = str(fChar*((indent1+TAB_0)+1)) + oLong[opt][1][desc] + "\n"
					else: cLine = str(fChar*((indent1+TAB_0)+1)) + oLong[opt][1][desc]
					cPrint += cLine
		else:
			cPrint += nMan
			cPrint += "\n\n"
			cPrint += str(fChar*indent0) + dMan
		cPrint += "\n\n\n"
	print (cPrint)

def parseOptions():
	# Collect all possible short option names
	cOptsShort = ""
	for i in range(len(OPTIONS_SHORT)):
		if len(OPTIONS_SHORT[i][0]) == 1: cOptsShort += OPTIONS_SHORT[i][0] + ","
		else: cOptsShort += OPTIONS_SHORT[i][0][0:1] + ":,"
	cOptsShort = cOptsShort.rstrip(",")
	# Collect all possible long option names
	cOptsLong = []
	for i in range(len(OPTIONS_LONG)): cOptsLong.append(OPTIONS_LONG[i][0])
	try:
		# Get current options
		opts, args = getopt.getopt(sys.argv[1:], cOptsShort, cOptsLong)
	except getopt.GetoptError:
		# If something went wrong, print error, help information and exit
		print ("\nSomething went wrong...\n")
		printUsage(globals()["SCRIPT_NAME"][0], globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["OPTIONS_LONG"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(2)
	
	# Exit if no input specified
	if len(sys.argv) == 1:
		print ("\n" + str(globals()["SCRIPT_NAME"][0]) + " needs at least one argument...\n")
		printUsage(globals()["SCRIPT_NAME"][0], globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["OPTIONS_LONG"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(0)
	
	if O_VERBOSE: print ("")

	# Parse long options
	for op, ar in opts:
		if op == "--regular":
			globals()["O_TRIM_L"] = True
			globals()["O_TRIM_T"] = True
			globals()["O_ALIGN"] = "left"
		if op == "--com-c":
			globals()["O_TRIM_L"] = True
			globals()["O_TRIM_T"] = True
			globals()["O_ALIGN"] = "left"
			globals()["O_COMMENT"] = "c"
		if op == "--com-python":
			globals()["O_TRIM_L"] = True
			globals()["O_TRIM_T"] = True
			globals()["O_ALIGN"] = "left"
			globals()["O_COMMENT"] = "python"

	# Parse short options
	for op, ar in opts:
		if op == "-h": globals()["O_HELP"] =         True
		if op == "-f": globals()["O_FILE"] =         True
		if op == "-s": globals()["O_SPACE_HOR"] =    int(ar)
		if op == "-S": globals()["O_SPACE_VER"] =    int(ar)
		if op == "-t": globals()["O_TRIM_L"] =       True
		if op == "-T": globals()["O_TRIM_T"] =       True
		if op == "-a": globals()["O_ALIGN"] =        str(ar)
		if op == "-c": globals()["O_FRAME"] =        str(ar[:1])
		if op == "-m": globals()["O_COMMENT"] =      str(ar)
		if op == "-v":
			globals()["O_VERBOSE"] =                   True
			print ("\nEntering Verbose Mode...\n")

def getLongest(inputLines):
	cWidth = 0
	cIndex = 0
	cContent = ""
	for i in range(len(inputLines)):
		cLine = inputLines[i].rstrip("\n")
		if len(cLine) > cWidth:
			cWidth = len(cLine)
			cIndex = i
			cContent = cLine
	result = (cWidth, cIndex, cContent)
	return (result)

def getCenterJust(lineLength, contentLength):
	cRem = lineLength - contentLength
	cLeft = cRem // 2
	cRight = (cRem // 2) + cRem % 2
	return ((cLeft, cRight))

def trimLines(inputLines, side):
	oLines = []
	if side == "left":
		for line in inputLines: oLines.append(line.lstrip())
	if side == "right":
		for line in inputLines: oLines.append(line.rstrip())
	return (oLines)

def makeComment(inputLines, commentType):
	oLines = []
	if commentType == "c":
		oLines.append("/**")
		for i in range(len(inputLines)): oLines.append(" * " + inputLines[i])
		oLines.append("*/")
	if commentType == "python":
		for i in range(len(inputLines)): oLines.append("# " + inputLines[i])
	return (oLines)

def makeLines(inputLines, longestLine):
	# Define result lines
	cLines = []
	# Get longest line's parameters
	cWidth = longestLine[0]
	cIndex = longestLine[1]
	cContent = longestLine[2]
	# Calculate full frame width and heigth (including frame characters)
	frameW = cWidth + 2 + (2*O_SPACE_HOR)
	frameH = len(inputLines) + 2 + (2*O_SPACE_VER)
	# Define top and bottom frame border
	endRow = frameW*O_FRAME
	# Define empty (padding) lines (e.g. between top border and top text row)
	paddingRow = O_FRAME + str((frameW-2)*" ") + O_FRAME
	# Insert top frame border
	cLines.append(endRow)
	# Insert top padding rows
	for i in range(O_SPACE_VER): cLines.append(paddingRow)
	# Handle content
	for i in range(len(inputLines)):
		# Define current raw line
		cLine = inputLines[i].rstrip("\n")
		# Define left padding
		cPadL = O_FRAME + " "*O_SPACE_HOR
		# Define right padding
		cPadR = " "*O_SPACE_HOR + O_FRAME
		# Define content (including filling space characters)
		if O_ALIGN == "left": cCont = cLine.ljust(cWidth)
		if O_ALIGN == "right": cCont = cLine.rjust(cWidth)
		if O_ALIGN == "center":
			cJust = getCenterJust(cWidth, len(cLine))
			cCont = " "*cJust[0] + cLine + " "*cJust[1]
		# Define current row
		cRow = cPadL + cCont + cPadR
		# Build current line
		cLines.append(cRow)
	# Insert bottom padding rows
	for i in range(O_SPACE_VER): cLines.append(paddingRow)
	# Insert bottom frame border
	cLines.append(endRow)
	# Return whole framed text
	return (cLines)

def main():
	# Parse options
	parseOptions()

	if O_VERBOSE:
		print ("Print this message and exit (-h): ".ljust(80, ".") + str(O_HELP))
		print ("Take input as file (-f): ".ljust(80, ".") + str(O_FILE))
		print ("Horizontal space between text and frame (-s <number>): ".ljust(80, ".") + str(O_SPACE_HOR))
		print ("Vertical space between text and frame (-S <number>): ".ljust(80, ".") + str(O_SPACE_VER))
		print ("Trim leading spaces (-t): ".ljust(80, ".") + str(O_TRIM_L))
		print ("Trim trailing spaces (-T): ".ljust(80, ".") + str(O_TRIM_T))
		print ("Align text block <direction> (left, right, center) (-a <direction>): ".ljust(80, ".") + str(O_ALIGN))
		print ("Frame character (-c): ".ljust(80, ".") + str(O_FRAME))
		print ("Create multiline comment block in <language> (c, python) (-m <language>): ".ljust(80, ".") + str(O_COMMENT))
		print ("Verbose mode (-v): ".ljust(80, ".") + str(O_VERBOSE))

	# Exit if help requested
	if O_HELP:
		printUsage(globals()["SCRIPT_NAME"][0], globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["OPTIONS_LONG"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(0)
	
	# Define input
	if O_FILE:
		try:
			INPUT_STRING_OBJ = open(INPUT_STRING)
			INPUT_STRING_LINES = INPUT_STRING_OBJ.readlines()
		except:
			print ("Cannot open " + INPUT_STRING)
			sys.exit(2)
	else:
		INPUT_STRING_OBJ = INPUT_STRING
		INPUT_STRING_LINES = INPUT_STRING_OBJ.splitlines()
	
	# Trim input lines if needed
	if O_TRIM_L: INPUT_STRING_LINES = trimLines(INPUT_STRING_LINES, "left")
	if O_TRIM_T: INPUT_STRING_LINES = trimLines(INPUT_STRING_LINES, "right")

	# Run Filter
	resultLines = makeLines(INPUT_STRING_LINES, getLongest(INPUT_STRING_LINES))

	# Make comment from OUTPUT IF NEEDED
	if O_COMMENT != False: resultLines = makeComment(resultLines, O_COMMENT)

	# Print result
	if O_VERBOSE:
		print ("\n\n##########")
		print ("#        #")
		print ("# RESULT #")
		print ("#        #")
		print ("##########\n")
	print ("\n".join(resultLines))
	if O_VERBOSE: print ("")

# ########
# # MAIN #
# ########

main ()
